# -*- coding: utf-8 -*-
#
# Univention Mail Cyrus
#  listener module: creating mailboxes and sieve scripts
#
# Copyright (C) 2004-2014 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = ''  # workaround for PEP 366
import listener
import univention.debug
import univention.uldap

import os
import string
import pwd
import grp
import subprocess

name='cyrus'
description='Create default imap folders'
filter='(&(objectClass=univentionMail)(uid=*))'
attributes=['uid', 'mailPrimaryAddress']


def is_groupware_user(new):
	if new.has_key('objectClass'):
		for oc in new['objectClass']:
			if oc.lower() == 'oxuserobject':
				return True
	return False


def create_cyrus_userlogfile(mailaddress):
	userlogfiles = listener.baseConfig.get('mail/cyrus/userlogfiles')
	if userlogfiles and userlogfiles.lower() in ['true', 'yes']:
		path='/var/lib/cyrus/log/%s' % (mailaddress)

		cyrus_id=pwd.getpwnam('cyrus')[2]
		mail_id=grp.getgrnam('mail')[2]

		if not os.path.exists( path ):
			os.mkdir(path)
		os.chmod(path,0750)
		os.chown(path,cyrus_id, mail_id)


def create_cyrus_mailbox(new):
	if new.has_key('mailPrimaryAddress') and new['mailPrimaryAddress'][0]:
		try:
			listener.setuid(0)
			p = os.popen('/usr/sbin/univention-cyrus-mkdir %s' % (string.lower(new['mailPrimaryAddress'][0])))
			p.close()
			create_cyrus_userlogfile(string.lower(new['mailPrimaryAddress'][0]))
		finally:
			listener.unsetuid()


def handler(dn, new, old):
	listener.setuid(0)
	try:
		ldapCon = univention.uldap.getMachineConnection(ldap_master = False)
	finally:
		listener.unsetuid()

	ldapbase = listener.baseConfig['ldap/base']
	fqdn = '%s.%s' % (listener.baseConfig['hostname'], listener.baseConfig['domainname'])
	mailboxrename = listener.baseConfig.get('mail/cyrus/mailbox/rename')

	# new imap server
	newHomeServer = ""
	if new and new.get("mailPrimaryAddress"):
		newDomain = new['mailPrimaryAddress'][0].split('@')[1]
		filter = '(&(cn=%s)(objectClass=oxMailDomainServerSettings))' % newDomain
		result = ldapCon.search(base="cn=domain,cn=mail,%s" % ldapbase, filter=filter, attr=['oxImapServer'])
		if result:
			newHomeServer = result[0][1].get("oxImapServer", [""])[0]

	# old imap server
	oldHomeServer = ""
	if old and old.get("mailPrimaryAddress"):
		oldDomain = old['mailPrimaryAddress'][0].split('@')[1]
		filter = '(&(cn=%s)(objectClass=oxMailDomainServerSettings))' % oldDomain
		result = ldapCon.search(base="cn=domain,cn=mail,%s" % ldapbase, filter=filter, attr=['oxImapServer'])
		if result:
			oldHomeServer = result[0][1].get("oxImapServer", [""])[0]

	# create mailbox on new home server
	if new and newHomeServer and is_groupware_user(new):
		# ignore changes if we are not the imap home server
		if not newHomeServer == fqdn:
			return

		# new account
		if not old:
				create_cyrus_mailbox(new)
		# modification
		else:
			# create mailbox if oldhome server was not this host
			# and mailboxrename is activated
			if oldHomeServer and mailboxrename and mailboxrename.lower() in ['true', 'yes']:
				if oldHomeServer and oldHomeServer != fqdn:
					# imap home server has changed, create a new mailbox
					create_cyrus_mailbox(new)
				else:
					# cyrus-mailboxrename.py will take care of this case
					pass
			else:
				create_cyrus_mailbox(new)
